<?php

namespace Ecommerce\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use PHPExcel_IOFactory;

class MeusProdutosController extends \Zeedhi\Framework\Controller\Report {
    
    protected $reportService;
    protected $birtStrategy;
    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $connection;
    protected $meusProdutosService;
    protected $dataSourceManager;
    protected $zhFileUploader;
    protected $fileServerURL;
    protected $entityManager;
    protected $imgPathURL;
    protected $uploadRoutes;
    
    
    public function __construct($reportService, $environment, $crudService, $birtStrategy,  $meusProdutosService, $connection, $dataSourceManager, $zhFileUploader, $fileServerURL, EntityManager $entityManager, $imgPathURL, $uploadRoutes) {
		$this->reportService     = $reportService;
		$this->crudService  = $crudService;
		$this->environment  = $environment;
        $this->birtStrategy      = $birtStrategy;
        $this->MeusProdutosService      = $meusProdutosService;
        $this->connection   = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->zhFileUploader    = $zhFileUploader;
        $this->fileServerURL     = $fileServerURL;
        $this->entityManager = $entityManager;
        $this->imgPathURL = $imgPathURL;
        $this->uploadRoutes     = $uploadRoutes;
        
	}
	
	protected $reportMapping = array (
        'ECO00102' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CDFORNECED"     => "CDFORNECED",
                        "NRORG"          => "NRORG",
                        "EXIBEPRODDESC"  => "EXIBEPRODDESC"
                    )
            )
    );
    
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrorgTrab();
        $qr2type = 'xlsx';
        $this->birtStrategy->setReportFormat($qr2type);
    }
	
	public function produtosPorForneced(DTO\Request $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("produtosPorForneced");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
	}
	
	private function setImgFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["FOTOPROD"] !== null) {
				$row["FOTOPROD"] = $this->imgPathURL.$row["FOTOPROD"];
			}
		}
		return new DataSet("produtosPorForneced", $rows);
	}

	public function saveImgDescriptionProd(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		try {
            $this->entityManager->beginTransaction();
            if (isset($row["IMAGE_UPLOAD"]) && isset($row["IMAGE_UPLOAD"]["0"]) 
    		&& isset($row["IMAGE_UPLOAD"]["0"]["path"]) && $row["IMAGE_UPLOAD"]["0"]["path"] !== null) {
    			$row["DSIDFOTOFILESERV"] = $row["IMAGE_UPLOAD"]["0"]["path"];
    			$fotoProd = $this->imgPathURL.$row["DSIDFOTOFILESERV"];
    		}else{
    		    if($row['FOTOPROD'] != null){
    		        $fotoProd = $row['FOTOPROD'];
    		    }else{
    		        $row["DSIDFOTOFILESERV"] = null;
    		        $fotoProd = null;
    		    }
    		    $fotoProd = $row['FOTOPROD'];
    		}
    		$this->MeusProdutosService->saveImgDescriptionProd($row);
    		$dataset = array(
                "PATHIMAGE"   => $fotoProd,
                "DSPRODUTO"   => $row['DSPRODUTO']
            );
            $response->addDataSet(new DataSet('data', $dataset));
		    $this->entityManager->commit();
        } catch (\Exception $e){
            $this->entityManager->rollBack();
            throw $e;
        }
	}
        
    public function savePhotosAutomaticZip(DTO\Request $request, DTO\Response $response)
    {
		$row = $request->getRow();
		$errorExtension = array();
		$errorNameArchive = array();
		$pathFolder = dirname(__DIR__, 1) .'/ImagesDownload/' . $row['CDFORNECED'] . mt_rand(10000, 99999);
		$pathZip = $pathFolder .'/Produtos.zip';
		
		mkdir($pathFolder, 0777);

		try {
		    $this->entityManager->beginTransaction();
            
            if (isset($row["UPLOAD_AUT"]) && isset($row["UPLOAD_AUT"]["0"]) 
    		    && isset($row["UPLOAD_AUT"]["0"]["path"]) && $row["UPLOAD_AUT"]["0"]["path"] !== null) {
    		    
    		    $path = $this->imgPathURL.$row["UPLOAD_AUT"]["0"]["path"];
    		    file_put_contents($pathZip, fopen($path, 'r'));
    		    
                $zip = new \ZipArchive;
                
                if ($zip->open($pathZip) === true) {
                    $zip->extractTo($pathFolder);
                    
                    for ($i = 0; $i < $zip->numFiles; $i++) {
                        $filename = $zip->getNameIndex($i);
                        $fileinfo = pathinfo($filename);
                        $extension = $fileinfo['extension'];
                        $basename = $fileinfo['basename'];

                        if ($extension === 'PNG' || $extension === 'png' || $extension === 'JPG' || $extension === 'jpg' || $extension === 'JPEG' || $extension ==='jpeg'){
                            $type = 'data:image/'.$extension;
                            $size = filesize($pathFolder.'/'.$basename);
                            $fileImageProd = file_get_contents("zip://".$pathZip."#".$filename);
                            $finalFileImageProd = $type.';base64,'.base64_encode($fileImageProd);
                            $fileRow = array(
                                'UPLOAD_AUT_IMAGE' => array(
                                    array(
                                        'b64File'            => $finalFileImageProd,
                                        "name"               => $basename,
                                        'type'               => 'image/'.$extension,
                                        'webkitRelativePath' => '',
                                        'size'               => ''
                                    )
                                )
                            );
                                    
                            $this->zhFileUploader->uploadFilesInRow($fileRow, $this->uploadRoutes['/savePhotosAutomatic']['fileField'], $this->uploadRoutes['/savePhotosAutomatic']['customFolder'] . '/' . $row['CDFORNECED']);
                            $updateProduct = $this->MeusProdutosService->savePhotosAutomatic($fileRow['UPLOAD_AUT_IMAGE'][0]['path'], $row['CDFORNECED'], $fileinfo['filename'], $row['CHECK_COD_CLIENTE']);
                            
                            if(!$updateProduct){
		                        array_push($errorNameArchive, $basename);
                            }
                        } else {
                            array_push($errorExtension, $basename);
                        }
                        
                        unlink($pathFolder.'/'.$basename);
                    }
                    
                    $zip->close();

                    $dataset = array(
                        "ERRORNAMEARCHIVE" => $errorNameArchive,
                        "ERROREXTENSION"    => $errorExtension
                    );

                    $response->addDataSet(new DataSet("data", $dataset));
                }
                
    		    $this->entityManager->commit();
    		}
        } catch (\Exception $e) {
            $this->entityManager->rollBack();
            throw $e;
        } finally {
            if (file_exists($pathZip)) {
                unlink($pathZip);
            }
            rmdir($pathFolder);
        }
    }
    
     public function importDescription(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$pathFolder = dirname(__DIR__, 1) .'/ImportTemp/' .$row['CDFORNECED'] . mt_rand(10000, 99999);

	    mkdir($pathFolder, 0777);
	    
		try {
		    $this->entityManager->beginTransaction();
            
            if (isset($row["EXCELIMPORT"]) && isset($row["EXCELIMPORT"]["0"]) 
    		    && isset($row["EXCELIMPORT"]["0"]["path"]) && $row["EXCELIMPORT"]["0"]["path"] !== null) {
    		    
    		    $path = $this->imgPathURL.$row["EXCELIMPORT"]["0"]["path"];
    		    $pathExcel = $pathFolder. '/'.$row["EXCELIMPORT"]["0"]["name"];
    		    
    		    file_put_contents($pathExcel, fopen($path, 'r'));
 
                $objPHPExcel = PHPExcel_IOFactory::load($pathExcel);
                $worksheet = $objPHPExcel->getSheet(0);
                $rowsSheet = array();
                foreach ($worksheet->getRowIterator() AS $rowSheet) {
                    $cellIterator = $rowSheet->getCellIterator();
                    $cellIterator->resetEnd('H');
                    $cellIterator->setIterateOnlyExistingCells(TRUE);
                    $cells = [];
                    $hasNotNullCell = false;
                    foreach ($cellIterator as $cell) {
                        $value = $cell->getValue();
                        $hasNotNullCell = $hasNotNullCell || $value;
                        $cells[] = $value;
                    }
                    if ($hasNotNullCell) {
                        $rowsSheet[] = $cells;
                    }
                }
                if (count($rowsSheet) < 2) {
                    unlink($pathExcel);
                    rmdir($pathFolder);
                    $response->addDataSet(new DataSet("error", array('A planilha encontra-se vazia!')));
                    throw new \Exception();
                }
                $validate = $this->validateColumns($rowsSheet, $pathFolder, $pathExcel, $response);
                if($validate){
                    $this->updateDescriptionProd($response, $row['CDFORNECED'], $rowsSheet);   
                }
                unlink($pathExcel);
                rmdir($pathFolder);
                $this->entityManager->commit();
                
    		}
		} catch (\Exception $e){
		    unlink($pathExcel);
            rmdir($pathFolder);
            $this->entityManager->rollBack();
            throw $e;
        }
     }
     
     public function validateColumns($rowsSheet, $pathFolder, $pathExcel, $response) {
        if (isset($rowsSheet[0])) {
            $header = $rowsSheet[0];
            if (count($header) == 3) {
                if($header[0] === 'CÓDIGO' && $header[1] === 'NOME' && $header[2] === 'DESCRIÇÃO'){
                    return true;
                }else{
                    unlink($pathExcel);
                    rmdir($pathFolder);
                    $response->addDataSet(new DataSet("error", array('O nome das colunas está inválido!')));
                    throw new \Exception();
                }
            } else {
                unlink($pathExcel);
                rmdir($pathFolder);
                $response->addDataSet(new DataSet("error", array('Número de colunas inválido!')));
                throw new \Exception();
            }
        } else {
            unlink($pathExcel);
            rmdir($pathFolder);
            $response->addDataSet(new DataSet("error", array('Modelo de planilha inválido!')));
            throw new \Exception();
        }
    }
    
    public function updateDescriptionProd ($response, $cdforneced, $rowsSheet){
        $errorCodProd = array();
        $errorColumnEmpty = array();
        array_shift($rowsSheet);
        foreach($rowsSheet as $row){
            if(count($row) === 3 && $row[0] != null & $row[2] != null){
                $updateDescription = $this->MeusProdutosService->updateDescriptionProd($cdforneced, $row[0], $row[2]);
                if(!$updateDescription){
                    $linha = array_search($row, $rowsSheet) + 2;
                    array_push($errorCodProd, $row[0] . ' na linha ' . $linha);
                }
            }else{
                $linha = array_search($row, $rowsSheet) + 2;
                array_push($errorColumnEmpty, $linha);
            }
        }
        $dataset = array(
                        "ERRORCODPROD" => $errorCodProd,
                        "ERRORCOLUMNEMPTY" => $errorColumnEmpty
                    );
        $response->addDataSet(new DataSet("data", $dataset));
    }
	
}